/*
famicom ROM cartridge utility - unagi
driver for anago

Copyright (C) 2009 ȯƱȹ

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "type.h"
#include "cemeleonusb/cusb.h"
#include "cemeleonusb/gpfw.h"

static void ag_address_set(HANDLE *h, long address, u8 port)
{
	u8 buf[4];
	int i = 0;
	buf[i++] = GPFW_WRITE | port;
	buf[i++] = address & 0xff;
	buf[i++] = GPFW_WRITE | (port + 1);
	buf[i++] = address >> 8;
	usb_bulk_write(h, GPFW_CPIPE, buf, i);
}

static u8 ag_cpu_data_get(HANDLE *h, int control)
{
	u8 buf[2];
	int i = 0;
	///ROMCS invert
	c = bit_invert(c, BITNUM_CPU_CONTROL_ROMCS);
	ag_cpu_control_set(csub_handle, c);
	buf[i++] = GPFW_READ | PORT_CPU_DATA;
	usb_bulk_write(h, GPFW_CPIPE, buf, i);
	usb_bulk_read(h, GPFW_RPIPE, buf, 1);
	return buf[0];
}

/*
δؿƤ databus  direction  OUTPUT ˤ뤳.
*/
static void ag_data_set(HANDLE *h, u8 port, u8 data)
{
	u8 buf[2];
	int i = 0;
	buf[i++] = GPFW_WRITE | port;
	buf[i++] = data;
	usb_bulk_write(h, GPFW_CPIPE, buf, i);
}

const u8 DIRECTION_CONTROL = (
	(DIRECTION_INPUT << PORT_CPU_DATA) |
	(DIRECTION_OUTPUT << PORT_CPU_CONTROL) |
	(DIRECTION_INPUT << PORT_PPU_DATA) |
	(DIRECTION_OUTPUT << PORT_PPU_CONTROL) |
	(DIRECTION_INPUT << PORT_VRAM_CONTROL) |
	0xe0 //bit5-7 write
);

static void ag_data_direction_set(HANDLE *h, int dir)
{
	u8 buf[2];
	u8 d = DIRECTION_CONTROL;
	int i = 0;
	buf[i++] = GPFW_WRITE | PORT_DIRECTION_CONTROL;
	
	switch(dir){
	case DIRECTION_WRITE: //=1
		d = bit_set(d, PORT_CPU_DATA);
		d = bit_set(d, PORT_PPU_DATA);
		break;
	case DIRECTION_READ: //=0
		d = bit_clear(d, PORT_CPU_DATA);
		d = bit_clear(d, PORT_PPU_DATA);
		break;
	default:
		assert(0);
	}
	buf[i++] = d;
	usb_bulk_write(h, GPFW_CPIPE, buf, i);
}

static const u8 CPU_CONTROL_STANDBY = (
	(0 << BITNUM_CPU_CONTROL_PHI2) |
	(1 << BITNUM_CPU_CONTROL_ROMCS) |
	(1 << BITNUM_CPU_CONTROL_RW)
);

static void ag_cpu_read(long address, long length, u8 *data)
{
	int c = CPU_CONTROL_STANDBY;
	if((address & ADDRESS_MASK_A15) == 0){
		c = bit_clear(BITNUM_CPU_CONTROL_ROMCS);
	}
	while(length != 0){
		ag_cpu_control_set(csub_handle, c);
		ag_address_set(csub_handle, address, PORT_CPU_ADDRESS_LOW);
		*data = ag_cpu_data_get(csub_handle, c);
		length--;
	}
}

static void ag_cpu_6502_write(long address, u8 *data)
{
	int c = CPU_CONTROL_STANDBY;
	//bus stop and dircetion set
	ag_data_direction_set(csub_handle, DIRECTION_WRITE);
	if((address & ADDRESS_MASK_A15) == 0){
		c = bit_clear(BITNUM_CPU_CONTROL_ROMCS);
	}
	ag_cpu_control_set(csub_handle, c);
	
	c = bit_clear(c, BITNUM_CPU_CONTROL_RW);
	ag_cpu_control_set(csub_handle, c);
	ag_cpu_address_set(csub_handle, address, PORT_CPU_ADDRESS_LOW);
	c = bit_set(c, BITNUM_CPU_CONTROL_PHI2);
	ag_cpu_control_set(csub_handle, c);
	
	ag_data_set(csub_handle, PORT_CPU_DATA, data);
	c = bit_clear(c, BITNUM_CPU_CONTROL_PHI2);
	ag_cpu_control_set(csub_handle, c);
}

